//some includes
#include "Live555Class.h"
#include "Live_UnicastMediaSubsession.h"

Live555Class::Live555Class()
{
    SetDefaultValues();
}


void Live555Class::SetDefaultValues()
{
    rtspServer = NULL;

    //Set default RTSP port
    RTSPPort = 554;

    //Set username and password to NULL
    RTSPUser[0]=0;
    RTSPPass[0]=0;

    watchForStop = 0;
}

Live555Class::~Live555Class()
{
}

void Live555Class::InitializeServer()
{
    if (rtspServer == NULL) {
        scheduler = BasicTaskScheduler::createNew();
        env = BasicUsageEnvironment::createNew(*scheduler);

        UserAuthenticationDatabase* authDB = NULL;

        //Create RTSP stream with or without username/password authentication
        if ((RTSPUser[0]==0x00) || (RTSPPass[0]==0x00)) {
            rtspServer = RTSPServer::createNew(*env, RTSPPort, NULL, 0);
        } else {
            authDB = new UserAuthenticationDatabase;
            authDB->addUserRecord(RTSPUser, RTSPPass);

            rtspServer = RTSPServer::createNew(*env, RTSPPort, authDB, 0);
        }

        // Increase the buffer size so we can handle high res streams..
        OutPacketBuffer::maxSize = 450000;

        // Set media session count to 0
        sessionCount = 0;
    }
}

void Live555Class::createMediaSession(FrameQueue *m_fqueue)
{
    char* url;
    ServerMediaSession* sms;

    if (rtspServer != NULL) {
        sms = ServerMediaSession::createNew(*env, m_fqueue->getStreamName(),
                                            m_fqueue->getStreamName(),
                                            m_fqueue->getStreamDesc());
        sms->addSubsession(UnicastMediaSubsession::createNew(*env, m_fqueue,
                                                             m_queue->getBitrate()));

        rtspServer->addServerMediaSession(sms);
        url = rtspServer->rtspURL(sms);
        *env << "Play this stream using the URL \"" << url << "\"\n";
        delete url;
    }
}

void Live555Class::Start()
{
    // If the server is initalised then exit
    if (rtspServer == NULL) {
        printf("rtspServer is NULL");
        return;
    }

    // The actual work is all carried out inside the LIVE555 Task scheduler
    printf("event loop called\n");
    env->taskScheduler().doEventLoop(&watchForStop); // does not return
    printf("event loop completed\n");


    Medium::close(rtspServer);

    // AM added, fix mem leaks ???
    env->reclaim();
    delete scheduler;

    return;
}

void Live555Class::Stop()
{
    watchForStop = 1;
}
