//==========================================================================
// Definitions.h allows you to select an encoder
//    - Choose between H264, H265, MP4, and MP2
//    - Choose between multicast and unicast
//==========================================================================


/**********
This library is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the
Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version. (See <http://www.gnu.org/copyleft/lesser.html>.)

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
more details.

You should have received a copy of the GNU Lesser General Public License
along with this library; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
**********/
// "liveMedia"
// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
// A template for a MediaSource encapsulating an audio/video input device
//
// NOTE: Sections of this code labeled "%%% TO BE WRITTEN %%%" are incomplete, and need to be written by the programmer
// (depending on the features of the particular device).
// Implementation

#include "Live_StreamSource.h"
#include <GroupsockHelper.hh> // for "gettimeofday()"
#include "FrameQueue.h"

unsigned StreamSource::referenceCount = 0;

StreamSource* StreamSource::createNew(UsageEnvironment& env, FrameQueue *Q_Source)
{
  return new StreamSource(env, Q_Source);
}

StreamSource::StreamSource(UsageEnvironment& env, FrameQueue *Q_Source)
               : FramedSource(env), Queue_Source(Q_Source)
{
  if (referenceCount == 0) {
    // Any global initialization of the device would be done here:

  }
  ++referenceCount;

  memset(&oldTime, 0, sizeof(struct timeval));
  startCheck = false;

  // Any instance-specific initialization of the device would be done here:

  /* register us with the Encoding thread so we'll get notices when new frame data turns up.. */
  Queue_Source->RegisterRTSP_Source(&(env.taskScheduler()), this);

  // We arrange here for our "deliverFrame" member function to be called
  // whenever the next frame of data becomes available from the device.
  //
  // If the device can be accessed as a readable socket, then one easy way to do this is using a call to
  //     envir().taskScheduler().turnOnBackgroundReadHandling( ... )
  // (See examples of this call in the "liveMedia" directory.)
  //
  // If, however, the device *cannot* be accessed as a readable socket, then instead we can implement is using 'event triggers':
  // Create an 'event trigger' for this device (if it hasn't already been done):
  eventTriggerId = envir().taskScheduler().createEventTrigger(deliverFrame0);
}

StreamSource::~StreamSource() {
  // Any instance-specific 'destruction' (i.e., resetting) of the device would be done here:

  /* de-register this source from the Encoding thread, since we no longer need notices.. */
  Queue_Source->Un_RegisterRTSP_Source();

  --referenceCount;
  if (referenceCount == 0) {
    // Any global 'destruction' (i.e., resetting) of the device would be done here:
  }

  // Reclaim our 'event trigger'
  envir().taskScheduler().deleteEventTrigger(eventTriggerId);
  eventTriggerId = 0;
}

unsigned StreamSource::GetRefCount() {
  return referenceCount;
}

void StreamSource::doGetNextFrame()
{
    // This function is called (by our 'downstream' object) when it asks for new data.
    // Note: If, for some reason, the source device stops being readable
    // (e.g., it gets closed), then you do the following:
    int status = Queue_Source->GetStatus();

    if (status == 2) {
        handleClosure(this);
        return;
    }

    // If a new frame of data is immediately available to be delivered, then do this now:
    if (status == 1) {
        deliverFrame();
    }

    // No new data is immediately available to be delivered.  We don't do anything more here.
    // Instead, our event trigger must be called (e.g., from a separate thread) when new data becomes available.
}

void StreamSource::deliverFrame0(void* clientData)
{
  ((StreamSource*)clientData)->deliverFrame();
}

void StreamSource::deliverFrame()
{
  // This function is called when new frame data is available from the device.
  // We deliver this data by copying it to the 'downstream' object, using the following parameters (class members):
  // 'in' parameters (these should *not* be modified by this function):
  //     fTo: The frame data is copied to this address.
  //         (Note that the variable "fTo" is *not* modified.  Instead,
  //          the frame data is copied to the address pointed to by "fTo".)
  //     fMaxSize: This is the maximum number of bytes that can be copied
  //         (If the actual frame is larger than this, then it should
  //          be truncated, and "fNumTruncatedBytes" set accordingly.)
  // 'out' parameters (these are modified by this function):
  //     fFrameSize: Should be set to the delivered frame size (<= fMaxSize).
  //     fNumTruncatedBytes: Should be set iff the delivered frame would have been
  //         bigger than "fMaxSize", in which case it's set to the number of bytes
  //         that have been omitted.
  //     fPresentationTime: Should be set to the frame's presentation time
  //         (seconds, microseconds).  This time must be aligned with 'wall-clock time' - i.e., the time that you would get
  //         by calling "gettimeofday()".
  //     fDurationInMicroseconds: Should be set to the frame's duration, if known.
  //         If, however, the device is a 'live source' (e.g., encoded from a camera or microphone), then we probably don't need
  //         to set this variable, because - in this case - data will never arrive 'early'.
  // Note the code below.

  if (!isCurrentlyAwaitingData()) {
	  return; // we're not ready for the data yet
  }

  unsigned char *data = NULL;

    fFrameSize = 0;
    unsigned int len = Queue_Source->getFrame(&data, &fPresentationTime);
    if (len != 0) {
        unsigned char *p = data;

        if (len >= 3 && p[0] == 0x00 && p[1] == 0x00 && p[2] == 0x01) {
            p += 3;
            len -= 3;
        }
        if (len >= 4 && p[0] == 0x00 && p[1] == 0x00 && p[2] == 0x00 && p[3] == 0x01) {
            p += 4;
            len -= 4;
        }

        //fprintf(stderr, ">> %u", len);
        //for (int s = 0; s < len && s < 14; s++)
        //    fprintf(stderr, " %0x", p[s]);
        //fprintf(stderr, "\n");
        if (fFrameSize + len > fMaxSize) {
            printf("Package Length execute the fMaxSize %u %u %u\n", fFrameSize, len, fMaxSize);
            free(data);
            return;
        }

        memmove(&fTo[fFrameSize], p, len);
        fFrameSize += len;
#if 0
        if (startCheck == true) {
            signed long long T2 = (100 * 1000ll) * ((signed long long)fPresentationTime.tv_sec) +
                          (signed long long)(fPresentationTime.tv_usec * 100/1000);
            signed long long T1 = (100 * 1000ll) * ((signed long long)oldTime.tv_sec) +
                          (signed long long)(oldTime.tv_usec * 100/1000);
            if (strcmp("stream_0", Queue_Source->getStreamName()) == 0) {
                printf("%lld\n", T2);
            }
            T2 = T2 - T1;
            if (T2 > 9100) {
                printf("%s, %lld\n", Queue_Source->getStreamName(), T2);
            }
        }
        oldTime = fPresentationTime;
        startCheck = true;
#endif
    } else {
        printf("Returning....\n");
        return;
    }

    if (data != NULL) {
        free(data);
    }

    //TODO: Close our stream on getFrame error
	//handleClosure(this);

  // After delivering the data, inform the reader that it is now available:
  FramedSource::afterGetting(this);
}
