#include "Live_UnicastMediaSubsession.h"
#include "FrameQueue.h"

#include <MPEG4ESVideoRTPSink.hh>
#include <MPEG4VideoStreamFramer.hh>
#include <MPEG4VideoStreamDiscreteFramer.hh>

#include <H265VideoRTPSink.hh>
#include <H265VideoStreamFramer.hh>
#include <H265VideoStreamDiscreteFramer.hh>

#include <H264VideoRTPSink.hh>
#include <H264VideoStreamFramer.hh>
#include <H264VideoStreamDiscreteFramer.hh>

#include <MPEG1or2VideoRTPSink.hh>
#include <MPEG1or2VideoStreamFramer.hh>
#include <MPEG1or2VideoStreamDiscreteFramer.hh>

UnicastMediaSubsession* UnicastMediaSubsession::createNew(UsageEnvironment& env,
                                                          FrameQueue *frmq,
                                                          unsigned int estimatedBitrate,
                                                          Boolean iFramesOnly,
                                                          double vshPeriod)
{
	return new UnicastMediaSubsession(env, frmq,
                                      estimatedBitrate,
                                      iFramesOnly, vshPeriod);
}

UnicastMediaSubsession::UnicastMediaSubsession(UsageEnvironment& env,
                                               FrameQueue *frmqueue,
                                               unsigned estimatedBitrate,
                                               Boolean iFramesOnly,
                                               double vshPeriod)
       : OnDemandServerMediaSubsession(env, True /*reuse the first source*/),
                                       fFrameQueue(frmqueue),
                                       fIFramesOnly(iFramesOnly),
                                       fVSHPeriod(vshPeriod)
{
	fEstimatedKbps = (estimatedBitrate + 500)/1000;
}

UnicastMediaSubsession::~UnicastMediaSubsession()
{
}


FramedSource* UnicastMediaSubsession::createNewStreamSource(unsigned /*clientSessionId*/,
                                                            unsigned& estBitrate)
{
	estBitrate = fEstimatedKbps;

	// Create a framer for the Video Elementary Stream:
	//LOG_MSG("Create Net Stream Source [%d]", estBitrate);
    FramedSource* videoSource = StreamSource::createNew(envir(), fFrameQueue);

	if (fFrameQueue->getEncoderType() == AV_CODEC_MPEG4) {
		return MPEG4VideoStreamDiscreteFramer::createNew(envir(), videoSource);
	}
	if (fFrameQueue->getEncoderType() == AV_CODEC_H265) {
		return H265VideoStreamDiscreteFramer::createNew(envir(), videoSource);
	}
	if (fFrameQueue->getEncoderType() == AV_CODEC_H264) {
		return H264VideoStreamDiscreteFramer::createNew(envir(), videoSource);
	}
}

RTPSink* UnicastMediaSubsession ::createNewRTPSink(Groupsock* rtpGroupsock,
                                                   unsigned char /*rtpPayloadTypeIfDynamic*/,
                                                   FramedSource* /*inputSource*/)
{
#if 0
	setVideoRTPSinkBufferSize();
#endif

	if (fFrameQueue->getEncoderType() == AV_CODEC_MPEG4) {
		return MPEG4ESVideoRTPSink::createNew(envir(), rtpGroupsock, 96);
	}
	if (fFrameQueue->getEncoderType() == AV_CODEC_H265) {
		return H265VideoRTPSink::createNew(envir(), rtpGroupsock, 96);
	}
	if (fFrameQueue->getEncoderType() == AV_CODEC_H264) {
		return H264VideoRTPSink::createNew(envir(), rtpGroupsock, 96);
	}
}

